importFrom("parallel",
           clusterApply, makeCluster, mclapply, stopCluster)
importFrom("grDevices", "dev.new", "grey")
importFrom("graphics", "plot", "legend", "lines")
importFrom("stats", "cov", "dnorm", "integrate", "median", "pnorm",
           "quantile", "rnorm", "runif", "sd", "uniroot")
importFrom("utils",
           "combn",
           "download.file",
           "flush.console",
           "read.table",
           "setTxtProgressBar",
           "txtProgressBar",
           "unzip")


export(
    CPPI,
    DEopt,
    EuropeanCall,
    EuropeanCallBE,
    French,
    GAopt,
    LS.info,
    LSopt,
    MA,
    NS,
    NSS,
    NSSf,
    NSf,
    PBO,
    PSopt,
    Ritter,
    SA.info,
    SAopt,
    Shiller,
    TA.info,
    TAopt,
    approxBondReturn,
    barrierOptionEuropean,
    bracketing,
    bundFuture,
    bundFutureImpliedRate,
    callCF,
    callHestoncf,
    callMerton,
    cfBSM,
    cfBates,
    cfHeston,
    cfMerton,
    cfVG,
    changeInterval,
    colSubset,
    convexity,
    divRatio,
    drawdown,
    duration,
    gbb,
    gbm,
    greedySearch,
    gridSearch,
    maxSharpe,
    minCVaR,
    minMAD,
    minvar,
    mvFrontier,
    mvPortfolio,
    pm,
    putCallParity,
    qTable,
    randomReturns,
    repairMatrix,
    resampleC,
    restartOpt,
    showChapterNames,
    showExample,
    tfAckley,
    tfEggholder,
    tfGriewank,
    tfRastrigin,
    tfRosenbrock,
    tfSchwefel,
    tfTrefethen,
    trackingPortfolio,
    vanillaBond,
    vanillaOptionAmerican,
    vanillaOptionEuropean,
    vanillaOptionImpliedVol,
    xtContractValue,
    xtTickValue,
    xwGauss,
    ytm
)

S3method(plot, TAopt)
S3method(print, DEopt)
S3method(print, LSopt)
S3method(print, PSopt)
S3method(print, TAopt)
