The file NMOFman.R contains the code from the 
NMOF manual. 

The file NMOFex.R contains the code from the report  
'Examples and Extensions for the NMOF package' (but 
this report was superseded by the NMOF manual).

The file NMOFdist.R contains the code from the report  
'Distributed computations with the NMOF package'. 

The pdf-versions of the reports are available from  
http://enricoschumann.net/files/NMOFman.pdf
http://enricoschumann.net/files/NMOFdist_Windows.pdf
http://enricoschumann.net/files/NMOFdist_Linux.pdf
